/*******************************************************************************
* File Name: FRAM_I2C_I2C.c
* Version 2.0
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "FRAM_I2C_I2C_PVT.h"

#if(FRAM_I2C_I2C_SCB_MODE_I2C_INC)
    #include "FRAM_I2C_I2C_I2C_PVT.h"
#endif /* (FRAM_I2C_I2C_SCB_MODE_I2C_INC) */

#if(FRAM_I2C_I2C_SCB_MODE_EZI2C_INC)
    #include "FRAM_I2C_I2C_EZI2C_PVT.h"
#endif /* (FRAM_I2C_I2C_SCB_MODE_EZI2C_INC) */

#if(FRAM_I2C_I2C_SCB_MODE_SPI_INC || FRAM_I2C_I2C_SCB_MODE_UART_INC)
    #include "FRAM_I2C_I2C_SPI_UART_PVT.h"
#endif /* (FRAM_I2C_I2C_SCB_MODE_SPI_INC || FRAM_I2C_I2C_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 FRAM_I2C_I2C_scbMode = FRAM_I2C_I2C_SCB_MODE_UNCONFIG;
    uint8 FRAM_I2C_I2C_scbEnableWake;
    uint8 FRAM_I2C_I2C_scbEnableIntr;

    /* I2C configuration variables */
    uint8 FRAM_I2C_I2C_mode;
    uint8 FRAM_I2C_I2C_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * FRAM_I2C_I2C_rxBuffer;
    uint8  FRAM_I2C_I2C_rxDataBits;
    uint32 FRAM_I2C_I2C_rxBufferSize;

    volatile uint8 * FRAM_I2C_I2C_txBuffer;
    uint8  FRAM_I2C_I2C_txDataBits;
    uint32 FRAM_I2C_I2C_txBufferSize;

    /* EZI2C configuration variables */
    uint8 FRAM_I2C_I2C_numberOfAddr;
    uint8 FRAM_I2C_I2C_subAddrSize;
#endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/

uint8 FRAM_I2C_I2C_initVar = 0u;

#if !defined (CY_REMOVE_FRAM_I2C_I2C_CUSTOM_INTR_HANDLER)
    cyisraddress FRAM_I2C_I2C_customIntrHandler = NULL;
#endif /* !defined (CY_REMOVE_FRAM_I2C_I2C_CUSTOM_INTR_HANDLER) */


/***************************************
*    Private Function Prototypes
***************************************/

static void FRAM_I2C_I2C_ScbEnableIntr(void);
static void FRAM_I2C_I2C_ScbModeStop(void);


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_Init
********************************************************************************
*
* Summary:
*  Initializes the SCB component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCB_I2CInit, SCB_SpiInit, SCB_UartInit or SCB_EzI2CInit.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_Init(void)
{
#if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
    if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        FRAM_I2C_I2C_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif(FRAM_I2C_I2C_SCB_MODE_I2C_CONST_CFG)
    FRAM_I2C_I2C_I2CInit();

#elif(FRAM_I2C_I2C_SCB_MODE_SPI_CONST_CFG)
    FRAM_I2C_I2C_SpiInit();

#elif(FRAM_I2C_I2C_SCB_MODE_UART_CONST_CFG)
    FRAM_I2C_I2C_UartInit();

#elif(FRAM_I2C_I2C_SCB_MODE_EZI2C_CONST_CFG)
    FRAM_I2C_I2C_EzI2CInit();

#endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_Enable
********************************************************************************
*
* Summary:
*  Enables the SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_Enable(void)
{
#if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if(!FRAM_I2C_I2C_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        FRAM_I2C_I2C_CTRL_REG |= FRAM_I2C_I2C_CTRL_ENABLED;

        FRAM_I2C_I2C_ScbEnableIntr();
    }
#else
    FRAM_I2C_I2C_CTRL_REG |= FRAM_I2C_I2C_CTRL_ENABLED;

    FRAM_I2C_I2C_ScbEnableIntr();
#endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZ I2C. Otherwise this function does not enable the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  FRAM_I2C_I2C_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void FRAM_I2C_I2C_Start(void)
{
    if(0u == FRAM_I2C_I2C_initVar)
    {
        FRAM_I2C_I2C_Init();
        FRAM_I2C_I2C_initVar = 1u; /* Component was initialized */
    }

    FRAM_I2C_I2C_Enable();
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component and its interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_Stop(void)
{
#if(FRAM_I2C_I2C_SCB_IRQ_INTERNAL)
    FRAM_I2C_I2C_DisableInt();
#endif /* (FRAM_I2C_I2C_SCB_IRQ_INTERNAL) */

    FRAM_I2C_I2C_CTRL_REG &= (uint32) ~FRAM_I2C_I2C_CTRL_ENABLED;  /* Disable scb IP */

#if(FRAM_I2C_I2C_SCB_IRQ_INTERNAL)
    FRAM_I2C_I2C_ClearPendingInt();
#endif /* (FRAM_I2C_I2C_SCB_IRQ_INTERNAL) */

    FRAM_I2C_I2C_ScbModeStop(); /* Calls scbMode specific Stop function */
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_SetRxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the RX FIFO to generate RX level interrupt.
*         The range of valid level values is between 0 and RX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = FRAM_I2C_I2C_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~FRAM_I2C_I2C_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (FRAM_I2C_I2C_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    FRAM_I2C_I2C_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_SetTxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has more entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the TX FIFO to generate TX level interrupt.
*         The range of valid level values is between 0 and TX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = FRAM_I2C_I2C_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~FRAM_I2C_I2C_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (FRAM_I2C_I2C_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    FRAM_I2C_I2C_TX_FIFO_CTRL_REG = txFifoCtrl;
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_SetCustomInterruptHandler
********************************************************************************
*
* Summary:
*  Registers a function to be called by the internal interrupt handler.
*  First the function that is registered is called, then the internal interrupt
*  handler performs any operation such as software buffer management functions
*  before the interrupt returns.  It is the user's responsibility not to break
*  the software buffer operations. Only one custom handler is supported, which
*  is the function provided by the most recent call.
*  At the initialization time no custom handler is registered.
*
* Parameters:
*  func: Pointer to the function to register.
*        The value NULL indicates to remove the current custom interrupt
*        handler.
*
* Return:
*  None
*
*******************************************************************************/
void FRAM_I2C_I2C_SetCustomInterruptHandler(cyisraddress func)
{
#if !defined (CY_REMOVE_FRAM_I2C_I2C_CUSTOM_INTR_HANDLER)
    FRAM_I2C_I2C_customIntrHandler = func; /* Register interrupt handler */
#else
    if(NULL != func)
    {
        /* Suppress compiler warning */
    }
#endif /* !defined (CY_REMOVE_FRAM_I2C_I2C_CUSTOM_INTR_HANDLER) */
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_ScbModeEnableIntr
********************************************************************************
*
* Summary:
*  Enables an interrupt for a specific mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void FRAM_I2C_I2C_ScbEnableIntr(void)
{
#if(FRAM_I2C_I2C_SCB_IRQ_INTERNAL)
    #if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
        /* Enable interrupt in NVIC */
        if(0u != FRAM_I2C_I2C_scbEnableIntr)
        {
            FRAM_I2C_I2C_EnableInt();
        }
    #else
        FRAM_I2C_I2C_EnableInt();

    #endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (FRAM_I2C_I2C_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: FRAM_I2C_I2C_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls the Stop function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void FRAM_I2C_I2C_ScbModeStop(void)
{
#if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
    if(FRAM_I2C_I2C_SCB_MODE_I2C_RUNTM_CFG)
    {
        FRAM_I2C_I2C_I2CStop();
    }
    else if (FRAM_I2C_I2C_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        FRAM_I2C_I2C_EzI2CStop();
    }
#if (!FRAM_I2C_I2C_CY_SCBIP_V1)
    else if (FRAM_I2C_I2C_SCB_MODE_UART_RUNTM_CFG)
    {
        FRAM_I2C_I2C_UartStop();
    }
#endif /* (!FRAM_I2C_I2C_CY_SCBIP_V1) */
    else
    {
        /* Do nothing for other modes */
    }
#elif(FRAM_I2C_I2C_SCB_MODE_I2C_CONST_CFG)
    FRAM_I2C_I2C_I2CStop();

#elif(FRAM_I2C_I2C_SCB_MODE_EZI2C_CONST_CFG)
    FRAM_I2C_I2C_EzI2CStop();

#elif(FRAM_I2C_I2C_SCB_MODE_UART_CONST_CFG)
    FRAM_I2C_I2C_UartStop();

#endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if(FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: FRAM_I2C_I2C_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void FRAM_I2C_I2C_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 hsiomSel [FRAM_I2C_I2C_SCB_PINS_NUMBER];
        uint32 pinsDm   [FRAM_I2C_I2C_SCB_PINS_NUMBER];

    #if (!FRAM_I2C_I2C_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!FRAM_I2C_I2C_CY_SCBIP_V1) */

        uint32 i;

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for(i = 0u; i < FRAM_I2C_I2C_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i]  = FRAM_I2C_I2C_HSIOM_DEF_SEL;
            pinsDm[i]    = FRAM_I2C_I2C_PIN_DM_ALG_HIZ;
        }

        if((FRAM_I2C_I2C_SCB_MODE_I2C   == mode) ||
           (FRAM_I2C_I2C_SCB_MODE_EZI2C == mode))
        {
            hsiomSel[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_I2C_SEL;
            hsiomSel[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_I2C_SEL;

            pinsDm[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_OD_LO;
            pinsDm[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_OD_LO;
        }
    #if (!FRAM_I2C_I2C_CY_SCBIP_V1)
        else if(FRAM_I2C_I2C_SCB_MODE_SPI == mode)
        {
            hsiomSel[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
            hsiomSel[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
            hsiomSel[FRAM_I2C_I2C_SCLK_PIN_INDEX]        = FRAM_I2C_I2C_HSIOM_SPI_SEL;

            if(FRAM_I2C_I2C_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;
                pinsDm[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsDm[FRAM_I2C_I2C_SCLK_PIN_INDEX]        = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;

            #if(FRAM_I2C_I2C_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
                pinsDm  [FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;
            #endif /* (FRAM_I2C_I2C_SS1_PIN) */

            #if(FRAM_I2C_I2C_MISO_SDA_TX_PIN)
                /* Disable input buffer */
                 pinsInBuf |= FRAM_I2C_I2C_MISO_SDA_TX_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_MISO_SDA_TX_PIN_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsDm[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;
                pinsDm[FRAM_I2C_I2C_SCLK_PIN_INDEX]        = FRAM_I2C_I2C_PIN_DM_STRONG;

            #if(FRAM_I2C_I2C_SS0_PIN)
                hsiomSel [FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
                pinsDm   [FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsInBuf                                |= FRAM_I2C_I2C_SS0_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_SS0_PIN) */

            #if(FRAM_I2C_I2C_SS1_PIN)
                hsiomSel [FRAM_I2C_I2C_SS1_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
                pinsDm   [FRAM_I2C_I2C_SS1_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsInBuf                                |= FRAM_I2C_I2C_SS1_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_SS1_PIN) */

            #if(FRAM_I2C_I2C_SS2_PIN)
                hsiomSel [FRAM_I2C_I2C_SS2_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
                pinsDm   [FRAM_I2C_I2C_SS2_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsInBuf                                |= FRAM_I2C_I2C_SS2_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_SS2_PIN) */

            #if(FRAM_I2C_I2C_SS3_PIN)
                hsiomSel [FRAM_I2C_I2C_SS3_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_SPI_SEL;
                pinsDm   [FRAM_I2C_I2C_SS3_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;
                pinsInBuf                                |= FRAM_I2C_I2C_SS3_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_SS2_PIN) */

                /* Disable input buffers */
            #if(FRAM_I2C_I2C_MOSI_SCL_RX_PIN)
                pinsInBuf |= FRAM_I2C_I2C_MOSI_SCL_RX_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_MOSI_SCL_RX_PIN) */

             #if(FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN)
                pinsInBuf |= FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN) */

            #if(FRAM_I2C_I2C_SCLK_PIN)
                pinsInBuf |= FRAM_I2C_I2C_SCLK_PIN_MASK;
            #endif /* (FRAM_I2C_I2C_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if(FRAM_I2C_I2C_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_UART_SEL;
                pinsDm  [FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if(0u != (FRAM_I2C_I2C_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_UART_SEL;
                    pinsDm  [FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;
                }

                if(0u != (FRAM_I2C_I2C_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_UART_SEL;
                    pinsDm  [FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;

                #if(FRAM_I2C_I2C_MISO_SDA_TX_PIN)
                     pinsInBuf |= FRAM_I2C_I2C_MISO_SDA_TX_PIN_MASK;
                #endif /* (FRAM_I2C_I2C_MISO_SDA_TX_PIN_PIN) */
                }

            #if !(FRAM_I2C_I2C_CY_SCBIP_V0 || FRAM_I2C_I2C_CY_SCBIP_V1)
                if(FRAM_I2C_I2C_UART_MODE_STD == subMode)
                {
                    if(0u != (FRAM_I2C_I2C_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                        hsiomSel[FRAM_I2C_I2C_SCLK_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_UART_SEL;
                        pinsDm  [FRAM_I2C_I2C_SCLK_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_DIG_HIZ;
                    }

                    if(0u != (FRAM_I2C_I2C_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                        hsiomSel[FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_HSIOM_UART_SEL;
                        pinsDm  [FRAM_I2C_I2C_SS0_PIN_INDEX] = FRAM_I2C_I2C_PIN_DM_STRONG;

                    #if(FRAM_I2C_I2C_SS0_PIN)
                        /* Disable input buffer */
                        pinsInBuf |= FRAM_I2C_I2C_SS0_PIN_MASK;
                    #endif /* (FRAM_I2C_I2C_SS0_PIN) */
                    }
                }
            #endif /* !(FRAM_I2C_I2C_CY_SCBIP_V0 || FRAM_I2C_I2C_CY_SCBIP_V1) */
            }
        }
    #endif /* (!FRAM_I2C_I2C_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if(FRAM_I2C_I2C_MOSI_SCL_RX_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_MOSI_SCL_RX_HSIOM_REG,
                                       FRAM_I2C_I2C_MOSI_SCL_RX_HSIOM_MASK,
                                       FRAM_I2C_I2C_MOSI_SCL_RX_HSIOM_POS,
                                       hsiomSel[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX]);

        FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_MOSI_SCL_RX_PIN_INDEX]);

    #if (!FRAM_I2C_I2C_CY_SCBIP_V1)
        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_INP_DIS,
                                     FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_MOSI_SCL_RX_PIN_MASK)));
    #endif /* (!FRAM_I2C_I2C_CY_SCBIP_V1) */
    #endif /* (FRAM_I2C_I2C_MOSI_SCL_RX_PIN) */

    #if(FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_HSIOM_REG,
                                       FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_HSIOM_MASK,
                                       FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_HSIOM_POS,
                                       hsiomSel[FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_wake_SetDriveMode((uint8)
                                                               pinsDm[FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_wake_INP_DIS,
                                     FRAM_I2C_I2C_spi_mosi_i2c_scl_uart_rx_wake_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN_MASK)));

         /* Set interrupt on falling edge */
        FRAM_I2C_I2C_SET_INCFG_TYPE(FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_INTCFG_REG,
                                        FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_INTCFG_TYPE_MASK,
                                        FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_INTCFG_TYPE_POS,
                                        FRAM_I2C_I2C_INTCFG_TYPE_FALLING_EDGE);
    #endif /* (FRAM_I2C_I2C_MOSI_SCL_RX_WAKE_PIN) */

    #if(FRAM_I2C_I2C_MISO_SDA_TX_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_MISO_SDA_TX_HSIOM_REG,
                                       FRAM_I2C_I2C_MISO_SDA_TX_HSIOM_MASK,
                                       FRAM_I2C_I2C_MISO_SDA_TX_HSIOM_POS,
                                       hsiomSel[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX]);

        FRAM_I2C_I2C_spi_miso_i2c_sda_uart_tx_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_MISO_SDA_TX_PIN_INDEX]);

    #if (!FRAM_I2C_I2C_CY_SCBIP_V1)
        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_miso_i2c_sda_uart_tx_INP_DIS,
                                     FRAM_I2C_I2C_spi_miso_i2c_sda_uart_tx_MASK,
                                    (0u != (pinsInBuf & FRAM_I2C_I2C_MISO_SDA_TX_PIN_MASK)));
    #endif /* (!FRAM_I2C_I2C_CY_SCBIP_V1) */
    #endif /* (FRAM_I2C_I2C_MOSI_SCL_RX_PIN) */

    #if(FRAM_I2C_I2C_SCLK_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_SCLK_HSIOM_REG, FRAM_I2C_I2C_SCLK_HSIOM_MASK,
                                       FRAM_I2C_I2C_SCLK_HSIOM_POS, hsiomSel[FRAM_I2C_I2C_SCLK_PIN_INDEX]);

        FRAM_I2C_I2C_spi_sclk_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_SCLK_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_sclk_INP_DIS,
                                     FRAM_I2C_I2C_spi_sclk_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_SCLK_PIN_MASK)));
    #endif /* (FRAM_I2C_I2C_SCLK_PIN) */

    #if(FRAM_I2C_I2C_SS0_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_SS0_HSIOM_REG, FRAM_I2C_I2C_SS0_HSIOM_MASK,
                                       FRAM_I2C_I2C_SS0_HSIOM_POS, hsiomSel[FRAM_I2C_I2C_SS0_PIN_INDEX]);

        FRAM_I2C_I2C_spi_ss0_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_SS0_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_ss0_INP_DIS,
                                     FRAM_I2C_I2C_spi_ss0_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_SS0_PIN_MASK)));
    #endif /* (FRAM_I2C_I2C_SS1_PIN) */

    #if(FRAM_I2C_I2C_SS1_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_SS1_HSIOM_REG, FRAM_I2C_I2C_SS1_HSIOM_MASK,
                                       FRAM_I2C_I2C_SS1_HSIOM_POS, hsiomSel[FRAM_I2C_I2C_SS1_PIN_INDEX]);

        FRAM_I2C_I2C_spi_ss1_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_SS1_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_ss1_INP_DIS,
                                     FRAM_I2C_I2C_spi_ss1_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_SS1_PIN_MASK)));
    #endif /* (FRAM_I2C_I2C_SS1_PIN) */

    #if(FRAM_I2C_I2C_SS2_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_SS2_HSIOM_REG, FRAM_I2C_I2C_SS2_HSIOM_MASK,
                                       FRAM_I2C_I2C_SS2_HSIOM_POS, hsiomSel[FRAM_I2C_I2C_SS2_PIN_INDEX]);

        FRAM_I2C_I2C_spi_ss2_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_SS2_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_ss2_INP_DIS,
                                     FRAM_I2C_I2C_spi_ss2_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_SS2_PIN_MASK)));
    #endif /* (FRAM_I2C_I2C_SS2_PIN) */

    #if(FRAM_I2C_I2C_SS3_PIN)
        FRAM_I2C_I2C_SET_HSIOM_SEL(FRAM_I2C_I2C_SS3_HSIOM_REG,  FRAM_I2C_I2C_SS3_HSIOM_MASK,
                                       FRAM_I2C_I2C_SS3_HSIOM_POS, hsiomSel[FRAM_I2C_I2C_SS3_PIN_INDEX]);

        FRAM_I2C_I2C_spi_ss3_SetDriveMode((uint8) pinsDm[FRAM_I2C_I2C_SS3_PIN_INDEX]);

        FRAM_I2C_I2C_SET_INP_DIS(FRAM_I2C_I2C_spi_ss3_INP_DIS,
                                     FRAM_I2C_I2C_spi_ss3_MASK,
                                     (0u != (pinsInBuf & FRAM_I2C_I2C_SS3_PIN_MASK)));
    #endif /* (FRAM_I2C_I2C_SS3_PIN) */
    }

#endif /* (FRAM_I2C_I2C_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (FRAM_I2C_I2C_CY_SCBIP_V0 || FRAM_I2C_I2C_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: FRAM_I2C_I2C_I2CSlaveNackGeneration
    ********************************************************************************
    *
    * Summary:
    *  Sets command to generate NACK to the address or data.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void FRAM_I2C_I2C_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (FRAM_I2C_I2C_CTRL_REG & FRAM_I2C_I2C_CTRL_EC_AM_MODE)) &&
            (0u == (FRAM_I2C_I2C_I2C_CTRL_REG & FRAM_I2C_I2C_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            FRAM_I2C_I2C_CTRL_REG &= ~FRAM_I2C_I2C_CTRL_EC_AM_MODE;
            FRAM_I2C_I2C_CTRL_REG |=  FRAM_I2C_I2C_CTRL_EC_AM_MODE;
        }

        FRAM_I2C_I2C_I2C_SLAVE_CMD_REG = FRAM_I2C_I2C_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (FRAM_I2C_I2C_CY_SCBIP_V0 || FRAM_I2C_I2C_CY_SCBIP_V1) */


/* [] END OF FILE */
