/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
*  This is the source code for example project for dev kit CY15FRAMKIT-001
*  The project communicates to I2C F-RAM device using I2C SCB.
*  It provides APIs to write and read data from F-RAM. The example project
*  includes basic write and read from a known location in F-RAM.
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation. All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/
#include <device.h>
#include <stdio.h>
#include "FRAM_I2C.h"

// Size of data buffer
#define BUFFER_SIZE            (16u)

// Size of string for UART output
#define STRING_SIZE            (50u)

// Example F-RAM Address 
#define EXAMPLE_ADDR_1         (0x2000)

// Example F-RAM Address 
#define EXAMPLE_ADDR_2         (0x3456)

// Example F-RAM Data 
#define EXAMPLE_DATA_BYTE      (0xa5)

// Macro for Communication Error
#define COM_ERR()  {UART_UartPutString("Communication Error !!\n\r"); for(;;){}}

// Data Copmarison Result macro    
#define FAIL ((uint8)0u)    
#define PASS ((uint8)1u) 

/*******************************************************************************
*  The main function calls F-RAM Read/Write APIs
*  1. Initialize data array
*  2. Init F-RAM I2C Block
*  3. Start-up Delay
*  4. Write EXAMPLE_DATA_BYTE to EXAMPLE_ADDR_1 address
*  5. Read a byte from address EXAMPLE_ADDR_1 and store in fram_rd_byte
*  6. Write 16 bytes of data from data_bytes array to EXAMPLE_ADDR_2 address
*  7. Read 15 bytes of data from EXAMPLE_ADDR_2 address through random read function
*  8. Read 16th byte through current read function
*  9. Compare the read data and glow BLUE LED if read is PASS
*******************************************************************************/
int main()
{
    uint8 data_bytes[BUFFER_SIZE];
	uint8 fram_rd_data[BUFFER_SIZE];
    uint8 fram_wr_byte;
    uint8 fram_rd_byte, fram_rd_current_byte;
	uint8 i;
    uint8 result;
    char  buf[STRING_SIZE];   
    uint32 addr;    

    // Enable global interrupts.
    CyGlobalIntEnable; 

    // Turn OFF Blue LED
    BLUE_LED_Write(1);
    
    // Initialize variables
	for(i = 0; i < BUFFER_SIZE; i++)
	{
	   data_bytes[i] = i+1;
	   fram_rd_data[i] = 0;
	}

    // Start UART Component
    UART_Start();

    while(UART_UartGetChar() != '\r')
    {
        UART_UartPutString("\rPress Enter to Run the PSoC4 I2C F-RAM Example Project     \r");
        CyDelay(500);
    }
    
    // Display Project Start 
    UART_UartPutString("\r------------------------------------------------------\n\r");    
    UART_UartPutString("\rCY15FRAMKIT-001 Kit - PSoC4 I2C F-RAM Example Project \n\r");
    UART_UartPutString("\r------------------------------------------------------\n\r");  
    
    // Initialize I2C F-RAM Component
	FRAM_I2C_Init();

    // F-RAM component initialized
    UART_UartPutString("I2C F-RAM Initialized\n\n\r");
    
    // F-RAM Write : Write 1 byte of data to F-RAM location EXAMPLE_ADDR_1
    addr = EXAMPLE_ADDR_1;
    fram_wr_byte = EXAMPLE_DATA_BYTE;
    if(FRAM_I2C_Write(FRAM_SLAVE_SRAM_ADDR, addr, &fram_wr_byte, 1) != FRAM_I2C_MSTR_NO_ERROR)
    {
        // I2C Error. loop forever
	    COM_ERR();
    }
    // Send the write data to UART
    sprintf(buf, "Data written to the address 0x%04X = 0x%02X\n\r", (uint16)addr, fram_wr_byte);
    UART_UartPutString(buf);    
    
    // F-RAM Read : Read 1 byte of data from F-RAM location EXAMPLE_ADDR_1
    addr = EXAMPLE_ADDR_1;    
    if(FRAM_I2C_Random_Read(FRAM_SLAVE_SRAM_ADDR, addr,&fram_rd_byte,1) != FRAM_I2C_MSTR_NO_ERROR)
    {
        // I2C Error. loop forever
	    COM_ERR();
    }
    // Send the read data to UART
    sprintf(buf, "Data read from the address 0x%04X = 0x%02X\n\r", (uint16)addr, fram_rd_byte);
    UART_UartPutString(buf);
    
    // F-RAM Write : Write 16 bytes from array data_bytes to F-RAM location EXAMPLE_ADDR_2
    addr = EXAMPLE_ADDR_2;        
    if(FRAM_I2C_Write(FRAM_SLAVE_SRAM_ADDR, addr, data_bytes, BUFFER_SIZE) != FRAM_I2C_MSTR_NO_ERROR)
    {
        // I2C Error. loop forever
	    COM_ERR();
    }
    // Send the written data to UART
    sprintf(buf, "\nData written to the address 0x%04X = \n\r", (uint16)addr);
    UART_UartPutString(buf);
    for(i=0;i<BUFFER_SIZE;i++)
    {
        sprintf(buf, "0x%02X\t", data_bytes[i]);
        UART_UartPutString(buf);        
    }
    
    // F-RAM Random Read : Read 15 bytes of data from SRAM starting from address EXAMPLE_ADDR_2. //
    //                     Data will be stored into fram_rd_data array.                          //
    if(FRAM_I2C_Random_Read(FRAM_SLAVE_SRAM_ADDR, EXAMPLE_ADDR_2,fram_rd_data,BUFFER_SIZE-1) != FRAM_I2C_MSTR_NO_ERROR)
    {
        // I2C Error. loop forever
	    COM_ERR();
    }
    // Send the read data to UART
    sprintf(buf, "\n\rRandom address read from the address 0x%04X = \n\r", (uint16)addr);
    UART_UartPutString(buf);
    for(i=0;i<(BUFFER_SIZE-1);i++)
    {
        sprintf(buf, "0x%02X\t", fram_rd_data[i]);
        UART_UartPutString(buf);        
    }
    
    // F-RAM Read : Read the 16th byte through current location read
    if(FRAM_I2C_Current_Read(FRAM_SLAVE_SRAM_ADDR, &fram_rd_current_byte,1) != FRAM_I2C_MSTR_NO_ERROR)
    {
        // I2C Error. loop forever
	    COM_ERR();
    }
    // Send the read data to UART
    sprintf(buf, "\n\rCurrent address read = 0x%02X\n\r",fram_rd_current_byte);
    UART_UartPutString(buf);
    
    result = PASS;  // Initialize result to PASS
    
    // Compare the single byte write and read data, current read data
    if((fram_rd_byte != fram_wr_byte) || (fram_rd_current_byte != data_bytes[BUFFER_SIZE-1]))
    {
        result = FAIL;  // F-RAM Status or memory Write/Read failed
    }
    
    // Compare the burst read data with written data
    for(i=0;i<(BUFFER_SIZE-1);i++)
    {
        if(fram_rd_data[i] != data_bytes[i])
        {
            result = FAIL;  // F-RAM Burst Write/Read failed
            break;
        }
    }
    
    // If PASS, turn ON PSoC4 BLUE LED
    if(result == PASS)
    {
        BLUE_LED_Write(0);  
        UART_UartPutString("\nPSoC4 I2C F-RAM Example Project - PASS\n\n\r");        
    }
    else
    {
        UART_UartPutString("\nPSoC4 I2C F-RAM Example Project - FAIL\n\r");                
    }

    // loop forever
	for(;;){}
    
}

